@@ -470,11 +470,10 @@ otherprog:
 		if (submitprotocol)
 		  fprintf(mfp, "with %s\n", submitprotocol);
 
-		if (!binary) {
-		  for (; optind < argc; ++optind)
-		    /*fprintf(mfp, "to %s\n", argv[optind]);*/
-		    check_and_print_to(mfp,argv[optind],notificationopt);
-		}
+		for (; optind < argc; ++optind)
+		  check_and_print_to(mfp,argv[optind],notificationopt);
+
+		fprintf(mfp,"env-end\n");
 
 		if (fflush(mfp) == EOF
 /* #ifdef NFSFSYNC */  /* This is propably ALWAYS a good idea.. */
@@ -491,8 +490,7 @@ otherprog:
 		 * which, though it may be unsatisfactory, is good for security.
 		 */
 		
-		if (binary) {	/* haa@cs.hut.fi  found a bug in read() --
-				   sizeof buf[0] != sizeof buf  :-)     */
+		if (binary) {
 		  while ((n = read(0, buf, sizeof buf)) > 0) {
 		    if (fwrite(buf, sizeof buf[0], n, mfp) != n)
 		      break;
@@ -681,8 +679,7 @@ check_and_print_to(mfp, addr, notify)
 
 	  printme = copy;
 	}
-	fprintf(mfp, "to %s\n", printme);
-	if (copy) free(copy);
+	/* FIRST 'todsn', THEN 'to' -header! */
 	fprintf(mfp, "todsn ORCPT=rfc822;");
 	s = printme;
 	while (*s) {
@@ -696,6 +693,8 @@ check_and_print_to(mfp, addr, notify)
 	if (notify)
 	  fprintf(mfp," NOTIFY=%s", notify);
 	putc('\n',mfp);
+	fprintf(mfp, "to %s\n", printme);
+	if (copy) free(copy);
 }
 
 #if 0
