@@ -211,6 +211,9 @@ extern RETSIGTYPE wantout __(());
 /* warning.c */
 extern int warning __(());
 
+/* skip821address.c */
+extern char *skip821address __((char *s));
+
 #ifndef CISTREQ
 #define  CISTREQ(x, y)      (cistrcmp((char *)(x), (char *)(y)) == 0)
 #define  CISTREQN(x, y, n)  (cistrncmp((char *)(x), (char *)(y), n) == 0)
