@@ -239,12 +239,12 @@ mail_open(type)
 
 	/* Extend when need! */
 
-	if (FILENO(fp) > mail_nfiles) {
+	if (FILENO(fp) >= mail_nfiles) {
 	  int nfile = FILENO(fp)+1;
 	  if (mail_file == NULL) {
 	    mail_file = (char**)mail_alloc((u_int)(sizeof(char*) * nfile));
 	  } else {
-	    mail_file = (char**)mail_realloc((u_int)(mail_file,sizeof(char*) * nfile));
+	    mail_file = (char**)mail_realloc(mail_file,(sizeof(char*) * nfile));
 	  }
 	  while (mail_nfiles < nfile) {
 	    mail_file[mail_nfiles] = NULL;
