@@ -200,7 +200,7 @@ extern int	s_apply __((int argc, char *argv[]));
 extern int	n_apply __((char **cpp, int argc, char *argv[]));
 extern struct header *hdr_rewrite __((u_char *name, struct header *h));
 extern void	setenvinfo __((struct envelope *e));
-extern char     *build_gensym __((int uid, char *type, char *DSNstr, char *errors_to));
+extern char     *build_gensym __((int uid, char *type, char *DSNstr, char *errors_to, char *sender));
 extern struct conscell *router __((struct address *a, int uid, char *type));
 extern struct conscell *crossbar __((struct conscell *from, struct conscell *to));
 
