@@ -1681,7 +1681,7 @@ sequencer(e, file)
 			/* Sender (and errto) level: */
 			for (nsp = rwp->down; nsp != NULL; nsp = nsp->next)
 				if (s_equal(nsp->info, sender1) &&
-				    s_equal1(rwp->errto,errto))
+				    s_equal1(nsp->errto,errto))
 					break;
 			if (nsp == NULL) {
 				nsp = rwalloc(&rwp->down);
@@ -1777,6 +1777,35 @@ sequencer(e, file)
 		fchown(FILENO(ofp), e->e_statbuf.st_uid, files_gid);
 		fchmod(FILENO(ofp), 0460);
 	}
+
+	FindEnvelope(eVerbose);
+	if (h != NULL
+	    && h->h_contents.a != NULL && h->h_contents.a->a_tokens != NULL) {
+		if (h->h_contents.a->a_tokens->p_tokens != NULL &&
+		    h->h_contents.a->a_tokens->p_tokens->t_type == String)
+			h->h_contents.a->a_tokens->p_tokens->t_type = Atom;
+		printToken(verbosefile, verbosefile + sizeof verbosefile,
+			   h->h_contents.a->a_tokens->p_tokens,
+			   (struct token *)NULL, 0);
+		/*
+		 * We have to be careful how we open this file, since one might
+		 * imagine someone trying to append to /etc/passwd using this
+		 * stuff.  The only safe way is to open it with the permissions
+		 * of the owner of the message file.
+		 */
+		setreuid(0, e->e_statbuf.st_uid);
+
+		if ((vfp = fopen(verbosefile, "a")) != NULL) {
+			fseek(vfp, (off_t)0, 2);
+			setvbuf(vfp, NULL, _IOLBF, 0);
+			fprintf(vfp, "router processed message %s\n", file);
+			fprintf(ofp, "%c%c%s\n", _CF_VERBOSE, _CFTAG_NORMAL,
+				verbosefile);
+		}
+		setreuid(0, 0);
+	} else
+		vfp = NULL;
+
 	fprintf(ofp, "%c%c%s\n",
 		_CF_MESSAGEID, _CFTAG_NORMAL, file);
 	fprintf(ofp, "%c%c%d\n",
@@ -1791,6 +1820,18 @@ sequencer(e, file)
 		fprintf(ofp, "%c%c%s\n",
 			_CF_LOGIDENT, _CFTAG_NORMAL, e->e_messageid);
 	/* else { we don't want to log anything } */
+	if (vfp != NULL) {
+	  if (envid != NULL)
+	    fprintf(vfp, "%c%c%s\n",
+		    _CF_DSNENVID, _CFTAG_NORMAL, envid);
+	  if (notaryret != NULL)
+	    fprintf(vfp, "%c%c%s\n",
+		    _CF_DSNRETMODE, _CFTAG_NORMAL, notaryret);
+	  if (e->e_messageid != NULL)
+	    fprintf(vfp, "%c%c%s\n",
+		    _CF_LOGIDENT, _CFTAG_NORMAL, e->e_messageid);
+	  /* else { we don't want to log anything } */
+	}
 	/*
 	 * If this message came from an error channel, then
 	 * do NOT produce an error message if something goes
@@ -1804,6 +1845,11 @@ sequencer(e, file)
                         putc(_CF_ERRORADDR, ofp);
                         putc(_CFTAG_NORMAL, ofp);
                         fprintf(ofp,"%s\n",errors_to);
+			if (vfp) {
+			  putc(_CF_ERRORADDR, vfp);
+			  putc(_CFTAG_NORMAL, vfp);
+			  fprintf(vfp,"%s\n",errors_to);
+			}
                         free(errors_to);
                         errors_to = NULL;
                 } else
@@ -1820,6 +1866,19 @@ sequencer(e, file)
 					putc(',', ofp);
 			}
 			putc('\n', ofp);
+			if (vfp) {
+			  putc(_CF_ERRORADDR, vfp);
+			  putc(_CFTAG_NORMAL, vfp);
+			  for (ap = h->h_contents.a;
+			       ap != NULL; ap = ap->a_next) {
+			    if (ap != h->h_contents.a)
+			      putc(' ', vfp);
+			    printAddress(vfp, ap->a_tokens, 0);
+			    if (ap->a_next != NULL)
+			      putc(',', vfp);
+			  }
+			  putc('\n', vfp);
+			}
 		}
 	}
 	/*
@@ -1856,34 +1915,6 @@ sequencer(e, file)
 		}
 	}
 
-	FindEnvelope(eVerbose);
-	if (h != NULL
-	    && h->h_contents.a != NULL && h->h_contents.a->a_tokens != NULL) {
-		if (h->h_contents.a->a_tokens->p_tokens != NULL &&
-		    h->h_contents.a->a_tokens->p_tokens->t_type == String)
-			h->h_contents.a->a_tokens->p_tokens->t_type = Atom;
-		printToken(verbosefile, verbosefile + sizeof verbosefile,
-			   h->h_contents.a->a_tokens->p_tokens,
-			   (struct token *)NULL, 0);
-		/*
-		 * We have to be careful how we open this file, since one might
-		 * imagine someone trying to append to /etc/passwd using this
-		 * stuff.  The only safe way is to open it with the permissions
-		 * of the owner of the message file.
-		 */
-		setreuid(0, e->e_statbuf.st_uid);
-
-		if ((vfp = fopen(verbosefile, "a")) != NULL) {
-			fseek(vfp, (off_t)0, 2);
-			setvbuf(vfp, NULL, _IOLBF, 0);
-			fprintf(vfp, "router processed message %s\n", file);
-			fprintf(ofp, "%c%c%s\n", _CF_VERBOSE, _CFTAG_NORMAL,
-				verbosefile);
-		}
-		setreuid(0, 0);
-	} else
-		vfp = NULL;
-
 	for (rwp = rwhead; rwp != NULL; rwp = rwp->next) {
 
 		for (nsp = rwp->down; nsp != NULL; nsp = nsp->next) {
@@ -1940,6 +1971,9 @@ sequencer(e, file)
 					prctladdr(rcp->info, vfp,
 						  _CF_RECIPIENT, "recipient");
 					putc('\n', vfp);
+					/* XX: DSN data output ! */
+					prdsndata(rcp->info, vfp,
+						  _CF_RCPTNOTARY, "recipient");
 				}
 			}
 		}
