@@ -25,6 +25,8 @@ extern char *strnsave();
 #endif
 extern int errno;
 
+extern int never_full_content; /* at conf.c */
+
 /*
  * There has to be some way of collecting the error messages produced during
  * delivery, and this is it. Each message is appended to the control file.
@@ -45,10 +47,28 @@ struct not {
 	char *rcpntp;
 };
 
+static void
+decodeXtext(fp,xtext)
+	FILE *fp;
+	char *xtext;
+{
+	for (;*xtext;++xtext) {
+	  if (*xtext == '+') {
+	    int c = '?';
+	    sscanf(xtext+1,"%02X",&c);
+	    putc(c,fp);
+	    if (*xtext) ++xtext;
+	    if (*xtext) ++xtext;
+	  } else
+	    putc(*xtext,fp);
+	}
+}
+
 static void	/* There is notaryreport() on transporters also.. */
-scnotaryreport(errfp,notary)
+scnotaryreport(errfp,notary,haserrsp)
 	FILE *errfp;
 	struct not *notary;
+	int *haserrsp;
 {
 	char *rcpt, *action, *status, *diagnostic, *wtt;
 	char const *typetag;
@@ -106,12 +126,18 @@ scnotaryreport(errfp,notary)
 	    typetag = type_local;
 	} else
 	  typetag = type_local;
-	if (notary->orcpt)
-	  fprintf(errfp, "Original-Recipient: %s\n", notary->orcpt);
+	if (notary->orcpt) {
+	  fprintf(errfp, "Original-Recipient: ");
+	  decodeXtext(errfp,notary->orcpt);
+	  putc('\n',errfp);
+	}
 	fprintf(errfp, "Final-Recipient: %s;%s\n", typetag, rcpt);
 	fprintf(errfp, "Action: %s\n", action);
-	if (status)
+	if (status) {
+	  if (*status == '4' || *status == '5')
+	    *haserrsp = 1;
 	  fprintf(errfp, "Status: %s\n", status);
+	}
 	fprintf(errfp, "Diagnostic-Code: %s\n", diagnostic);
 	if (wtt && wtt[0] != 0)
 	  fprintf(errfp, "Remote-MTA: %s\n", wtt);
@@ -163,6 +189,24 @@ msgerror(vp, offset, message)
 	fclose(fp);
 }
 
+void printenvaddr(fp,addr)
+FILE *fp;
+char *addr;
+{
+	char *s;
+	fprintf(fp, "todsn ORCPT=rfc822;");
+	s = addr;
+	for  (;*s;++s) {
+	  u_char c = *s;
+	  if ('!' <= c && c <= '~' && c != '+' && c != '=')
+	    putc(c,fp);
+	  else
+	    fprintf(fp,"+%02X",c);
+	}
+	putc('\n',fp);
+	fprintf(fp, "to <%s>\n", addr);
+}
+
 /* Pick recipient address from the input line.
    EXTREMELY Simple minded parsing.. */
 static void pick_env_addr(buf,mfp)
@@ -184,10 +228,10 @@ FILE *mfp;
 	  s = strrchr(buf,'>');
 	  if (s) *s = 0;
 	  else return; /* No trailing '>' ? BAD BAD! */
-	  fprintf(mfp,"to <%s>\n",buf);
+	  printenvaddr(mfp,buf);
 	} else {
 	  /*  Cc: some-address  */
-	  fprintf(mfp,"to <%s>\n",buf);
+	  printenvaddr(mfp,buf);
 	}
 }
 
@@ -213,6 +257,8 @@ reporterrs(cfpi)
 	struct ctlfile *cfp;
 	char *rcpntpointer;
 	int no_error_report = 0;
+	int has_errors = 0;
+	int lastoffset;
 	char path[MAXPATHLEN], buf[BUFSIZ], mpath[MAXPATHLEN];
 
 	if (cfpi->haderror == 0)
@@ -221,7 +267,7 @@ reporterrs(cfpi)
 		return;
 	no_error_report = cfpi->iserrmesg;
 
-	boundarystr = (char*)buildboundary("dlvryerr");
+	boundarystr = (char*)buildboundary("dlvryreport");
 
 	eaddr = cfpi->erroraddr;
 	deliveryform = cfpi->deliveryform;
@@ -256,6 +302,7 @@ reporterrs(cfpi)
 	if (cfp->mid) free(cfp->mid);
 	cfp->mid = midbuf; /* This is the original one! */
 	wroteheader = 0;
+	lastoffset = cfp->offset[cfp->nlines-1];
 	for (i = 0; i < cfp->nlines; ++i, ++lp) {
 	  cp = cfp->contents + *lp;
 
@@ -268,12 +315,16 @@ reporterrs(cfpi)
 	  cp = cfp->contents + *lp + 2;
 
 	  byteidx = atoi(cp);
+	  if (byteidx < 0 || byteidx > lastoffset)
+	    byteidx = 0; /* Invalid offset ?! */
 	  while (*cp  && isascii(*cp) && isdigit(*cp))
 	    ++cp;
 	  headeridx = -1;
 	  if (*cp == ':') {
 	    ++cp;
 	    headeridx = atoi(cp);
+	    if (headeridx > lastoffset)
+	      headeridx = 0;
 	    while (*cp && isascii(*cp) && (isdigit(*cp) || *cp == '-'))
 	      ++cp;
 	  }
@@ -281,11 +332,15 @@ reporterrs(cfpi)
 	  if (*cp == ':') {
 	    ++cp;
 	    drptidx = atoi(cp);
+	    if (drptidx > lastoffset)
+	      drptidx = -1;
 	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
 	      ++cp;
 	  }
 	  if (*cp == ':') {
 	    int offs = atoi(++cp);
+	    if (offs > lastoffset)
+	      offs = -1;
 	    if (offs > 0)
 	      dnsrecipient = cfp->contents + offs;
 	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
@@ -299,6 +354,9 @@ reporterrs(cfpi)
 	    ++cp;
 	  }
 	  rcpntpointer = cfp->contents + byteidx + 2 + _CFTAG_RCPTPIDSIZE;
+	  if (do_syslog)
+	    syslog(LOG_INFO, "%s: <%s>: %s", cfp->mid, rcpntpointer, cp);
+
 	  if (notary != NULL && *notary != 0) {
 	    if (!notaries) {
 	      notaryspc = 8;
@@ -373,7 +431,7 @@ reporterrs(cfpi)
 	    fprintf(errfp, "channel error\n");
 
 	    if (!no_error_report)
-	      fprintf(errfp, "to <%s>\n", eaddr);
+	      printenvaddr(errfp,eaddr);
 	    sprintf(path, "%s/%s/%s", mailshare, FORMSDIR,
 		    deliveryform ? deliveryform : "delivery");
 	    if ((fp = fopen(path, "r")) != NULL) {
@@ -450,10 +508,6 @@ reporterrs(cfpi)
 	  } else
 	    fprintf(errfp, "%s\n", cp);
 	  lockaddr(cfp->fd, NULL, *lp + 1, _CFTAG_NORMAL, _CFTAG_OK, mpath, "diagmsg");
-#ifdef	LOG_INFO
-	  syslog(LOG_INFO, "%s: <%s>: %s",
-		 cfp->mid, cfp->contents + byteidx + 2, cp);
-#endif				/* LOG_INFO */
 	}
 	
 	fprintf(errfp, "\n");
@@ -465,15 +519,18 @@ reporterrs(cfpi)
 	} else {
 	  fprintf(errfp, "Reporting-MTA: x-local-hostname; -unknown-\n");
 	}
-	if (envid != NULL)
-	  fprintf(errfp, "Original-Envelope-Id: %s\n",envid);
+	if (envid != NULL) {
+	  fprintf(errfp, "Original-Envelope-Id: ");
+	  decodeXtext(errfp,envid);
+	  putc('\n',errfp);
+	}
 	/* rfc822date() returns a string with trailing newline! */
 	fprintf(errfp, "Arrival-Date: %s", rfc822date(&cfpi->ctime));
 	fprintf(errfp, "\n");
 
 	/* Now scan 'em all again for IETF-NOTARY */
 	for (i = 0; i < notarycnt; ++i) {
-	  scnotaryreport(errfp, &notaries[i]);
+	  scnotaryreport(errfp, &notaries[i],&has_errors);
 	  if (notaries[i].not != NULL)
 	    free(notaries[i].not);
 	}
@@ -517,7 +574,8 @@ reporterrs(cfpi)
 	      fseek(fp,(off_t)0,0);
 	    }
 	  }
-	  if (!cfp->dsnretmode || CISTREQN(cfp->dsnretmode,"FULL",4)) {
+	  if (has_errors && !never_full_content &&
+	      (!cfp->dsnretmode || CISTREQN(cfp->dsnretmode,"FULL",4))) {
 	    /* Copy out the rest (=body) with somewhat more efficient method */
 	    while ((n = fread(buf, sizeof buf[0], sizeof buf, fp)) > 0)
 	      fwrite(buf, sizeof buf[0], n, errfp);
