@@ -18,7 +18,6 @@
 #include <errno.h>
 #include "scheduler.h"
 #include <sys/stat.h>
-#include "syslog.h"
 #include <fcntl.h>
 #include "mail.h"
 #include <string.h>
@@ -45,6 +44,7 @@
 #endif
 
 #include "prototypes.h"
+#include "syslog.h"
 
 #ifndef	MAXNAMLEN /* POSIX.1 ... */
 #define MAXNAMLEN NAME_MAX
@@ -74,7 +74,7 @@ extern char *postoffice;	/* At  mail_open()  */
 char	*rendezvous;
 char	*pidfile = PID_SCHEDULER;
 char	*mailshare;
-char	*log;
+char	*logfn;
 static int mustexit;
 static int gotalarm;
 static int dumpq;
@@ -82,6 +82,7 @@ static int canexit;
 static int rereadcf;
 static int dlyverbose = 0;
 time_t	sched_starttime;
+int	do_syslog = 0;
 int	verbose = 0;
 int	querysocket = -1;	/* fd of TCP socket to listen for queries */
 int	D_alloc = 0;
@@ -267,7 +268,7 @@ main(argc, argv)
 
 	global_maxkids = resources_query_nofiles()-10;
 
-	postoffice = rendezvous = log = config = NULL;
+	postoffice = rendezvous = logfn = config = NULL;
 	daemonflg = 1;
 	dlyverbose = 0;
 	syncstart = 0;
@@ -297,7 +298,7 @@ main(argc, argv)
 				++hashlevels;
 			break;
 		case 'L':	/* override default log file */
-			log = optarg;
+			logfn = optarg;
 			break;
 		case 'N':
 			if ((transportmaxnofiles = atoi(optarg)) < 10)
@@ -353,11 +354,11 @@ main(argc, argv)
 	  mailshare = MAILSHARE;
 	if ((cp = getzenv("LOGDIR")) != NULL)
 	  qlogdir = cp;
-	if (daemonflg && log == NULL) {
-	  log = emalloc(2 + (u_int)(strlen(qlogdir) + strlen(progname)));
-	  sprintf(log, "%s/%s", qlogdir, progname);
+	if (daemonflg && logfn == NULL) {
+	  logfn = emalloc(2 + (u_int)(strlen(qlogdir) + strlen(progname)));
+	  sprintf(logfn, "%s/%s", qlogdir, progname);
 	}
-	if (log != NULL) {
+	if (logfn != NULL) {
 	  /* loginit is a signal handler, so can't pass log */
 	  if (loginit(SIGHUP) < 0) /* do setlinebuf() there */
 	    die(1, "log initialization failure");
@@ -1156,7 +1157,7 @@ static int bcfcn(a, b)
      const void *a, *b;
 {
 	const struct offsort *aa = a, *bb = b;
-	return cistrcmp(bcp + aa->offset, bcp + bb->offset);
+	return strcmp(bcp + aa->offset, bcp + bb->offset);
 }
 
 
@@ -1206,7 +1207,7 @@ static struct ctlfile *vtxprep(cfp, file)
 	register int i, opcnt;
 	register long *lp;
 	int svn;
-	char *cp, *channel, *host, *l_channel, *l_host;
+	char *cp, *cp2, *channel, *host, *l_channel, *l_host;
 	char *echannel, *ehost, *l_echannel, *l_ehost, mfpath[100], flagstr[2];
 	char *latest_sender = NULL;
 	char *senderchannel = NULL;
@@ -1295,9 +1296,13 @@ static struct ctlfile *vtxprep(cfp, file)
 	      while (*cp != 0 && *cp != ' ') ++cp; 
 	      if (*cp == ' ') *cp++ = 0;
 	      while (*cp == ' ') ++cp;
-	      while (*cp != 0 && *cp != ' ') ++cp; while (*cp == ' ') ++cp;
+	      /* Scan over the sender 'host' */
+	      cp = skip821address(cp);
+	      while (*cp == ' ') ++cp;
+	      /* Scan over the sender 'user' */
 	      latest_sender = cp;
-	      while (*cp != 0 && *cp != ' ') ++cp;  *cp = 0; /* We assume: No spaces at the sender addr! */
+	      cp = skip821address(cp);
+	      *cp = 0;
 	      if (cfp->erroraddr)  free(cfp->erroraddr);
 	      cfp->erroraddr = strsave(latest_sender);
 	      if (strcmp(senderchannel,"error")==0)
@@ -1312,6 +1317,7 @@ static struct ctlfile *vtxprep(cfp, file)
 	      }
 	      offarr[opcnt].offset = *lp + 2;
 	      cp += 2;
+	      strlower(cp);
 	      if (*cp == ' ' || (*cp >= '0' && *cp <= '9')) {
 		/* New PID locking scheme.. */
 		offarr[opcnt].offset += _CFTAG_RCPTPIDSIZE;
@@ -1333,21 +1339,21 @@ static struct ctlfile *vtxprep(cfp, file)
 		while (*cp != 0) {
 		  while (*cp == ' ' || *cp == '\t') ++cp;
 		  if (CISTREQN("NOTIFY=",cp,7)) {
-		    offarr[opcnt].notifyflg = 0;
+		    offarr[prevrcpt].notifyflg = 0;
 		    cp += 7;
 		    while (*cp) {
 		      if (CISTREQN(cp,"NEVER",5)) {
 			cp += 5;
-			offarr[opcnt].notifyflg |= NOT_NEVER;
-		      } else if (CISTREQN(cp,"DELAY",7)) {
+			offarr[prevrcpt].notifyflg |= NOT_NEVER;
+		      } else if (CISTREQN(cp,"DELAY",5)) {
 			cp += 5;
-			offarr[opcnt].notifyflg |= NOT_DELAY;
+			offarr[prevrcpt].notifyflg |= NOT_DELAY;
 		      } else if (CISTREQN(cp,"FAILURE",7)) {
 			cp += 7;
-			offarr[opcnt].notifyflg |= NOT_FAILURE;
+			offarr[prevrcpt].notifyflg |= NOT_FAILURE;
 		      } else if (CISTREQN(cp,"SUCCESS",7)) {
 			cp += 7;
-			offarr[opcnt].notifyflg |= NOT_SUCCESS;
+			offarr[prevrcpt].notifyflg |= NOT_SUCCESS;
 		      } else {
 			break; /* Burp ?? */
 		      }
@@ -1531,7 +1537,6 @@ static struct ctlfile *vtxprep(cfp, file)
 	  if ((echannel = strchr(channel, ' ')) == NULL) /* error! */
 	    continue;
 	  *echannel = '\0';
-	  strlower(channel);
 	  host = echannel + 1;
 	  while (*host == ' ') ++host; /* Skip space */
 #if 1
@@ -1564,11 +1569,13 @@ static struct ctlfile *vtxprep(cfp, file)
 	      continue;		/* Not proper separator.. error! */
 	  } else
 #endif
-	    if ((ehost = strchr(host, ' ')) == NULL) /* error! */
-	      continue;
+	    {
+	      ehost = skip821address(host);
+	      if (ehost == NULL || ehost == host) /* error! */
+		continue;
+	    }
 
 	  *ehost = '\0';
-	  strlower(host);
 	  /* compare with the last ones */
 	  if (strcmp(channel, l_channel) || strcmp(host, l_host)) {
 	    /* wrap and tie the old vertex node */
@@ -1640,6 +1647,7 @@ static struct ctlfile *vtxprep(cfp, file)
 	  /* vp->sender = strsave(offarr[snv].sender); */
 	  vp->headeroffset = offarr[svn].headeroffset; /* Just any of them will do */
 	  vp->drptoffset = offarr[svn].drptoffset;
+	  vp->notaryflg  = offarr[svn].notifyflg;
 	  while (svn < i) {
 	    vp->index[i-svn-1] = offarr[svn].myidx;
 	    ++svn;
