@@ -234,9 +234,10 @@ unctlfile(cfp, no_unlink)
 
 	if (!no_unlink) {
 	  reporterrs(cfp);
-#ifdef	LOG_INFO
-	  syslog(LOG_INFO, "%s: done", cfp->mid);
-#endif	/* LOG_INFO */
+
+	  if (do_syslog)
+	    syslog(LOG_INFO, "%s: done", cfp->mid);
+
 	  eunlink(path);
 	  if (verbose)
 	    printf("%s: unlink %s (mid=0x%p)", cfp->logident, path, cfp->mid);
@@ -459,9 +460,8 @@ deletemsg(msgid, curcfp)
 	 * test for some 'return-receipt-requested' flag.
 	 */
 
-#ifdef	LOG_INFO
-	syslog(LOG_INFO, "%s: obsoleted by %s", cfp->mid, curcfp->mid);
-#endif	/* LOG_INFO */
+	if (do_syslog)
+	  syslog(LOG_INFO, "%s: obsoleted by %s", cfp->mid, curcfp->mid);
 
 	/*
 	 * unvertex() will do unctlfile() on the last vertex, hence
