@@ -1051,7 +1051,7 @@ char *buf, *cp;
 	  type(-250,NULL, "ENHANCEDSTATUSCODES");
 	  type(-250,NULL, "EXPN");
 	  type(-250,NULL, "VRFY");
-	  type(-250,NULL, "X-DSN"); /* XX: Change to "DSN" when all parts of it function! */
+	  type(-250,NULL, "DSN");
 	  type(-250,NULL, "X-RCPTLIMIT 10000"); /* VERY HIGH figure, normal is 100 */
 	  type(-250,NULL, "X-TURNME");
 	  type(-250,NULL, "ETRN");
@@ -1443,6 +1443,8 @@ char *buf, *cp;
 	  }
 	}
 
+	/* FIRST 'todsn', THEN 'to' -HEADER */
+
 	/* IETF-NOTARY DSN data: */
 	fputs("todsn",mfp);
 	if (drpt_notify) fprintf(mfp, " %s", drpt_notify);
@@ -2580,6 +2582,31 @@ router(function, holdlast, args)
 }
 
 
+void
+type821err(code,status,inbuf,msg,s1,s2,s3)
+	int code;
+	char *status, *msg, *inbuf;
+	void *s1, *s2, *s3; /* XX: Propably not portable.. */
+{
+	char *s = inbuf+3+strlen(status)+1;
+	int maxcnt = 200;
+
+	printf("%03d-%s ", code < 0 ? -code : code,status);
+	if (logfp != NULL)
+	  fprintf(logfp, "%dw\t%03d-%s ", pid, code,status);
+	while (s < rfc821_error_ptr && --maxcnt >= 0) {
+	  ++s;
+	  putc(' ',stdout);
+	  if (logfp != NULL)
+	    putc(' ',logfp);
+	}
+	printf("^\r\n");
+	if (logfp != NULL)
+	  fprintf(logfp,"^\n");
+	type(code,status,msg,s1,s2,s3);
+}
+
+
 #ifdef HAVE_VPRINTF
 #ifdef HAVE_STDARG_H
 void
@@ -2591,9 +2618,9 @@ type(int code, char *status, char *fmt
 #else
 /* VARARGS2 */
 void
-type(code, fmt, va_alist)
+type(code, status, fmt, va_alist)
 	int code;
-	char *fmt;
+	char *status, *fmt;
 	va_dcl
 #endif
 #else /* No VPRINTF */
@@ -2735,31 +2762,6 @@ type(code, status, fmt, s1, s2, s3, s4, s5, s6)
 #endif
 }
 
-
-void
-type821err(code,status,inbuf,msg,s1,s2,s3)
-	int code;
-	char *status, *msg, *inbuf;
-	void *s1, *s2, *s3; /* XX: Propably not portable.. */
-{
-	char *s = inbuf+3+strlen(status)+1;
-	int maxcnt = 200;
-
-	printf("%03d-%s ", code < 0 ? -code : code,status);
-	if (logfp != NULL)
-	  fprintf(logfp, "%dw\t%03d-%s ", pid, code,status);
-	while (s < rfc821_error_ptr && --maxcnt >= 0) {
-	  ++s;
-	  putc(' ',stdout);
-	  if (logfp != NULL)
-	    putc(' ',logfp);
-	}
-	printf("^\r\n");
-	if (logfp != NULL)
-	  fprintf(logfp,"^\n");
-	type(code,status,msg,s1,s2,s3);
-}
-
 static void header_to_mime __((char *, int *, int));
 static void header_to_mime(buf, lenptr, maxlen)
 	char *buf;
