@@ -167,37 +167,30 @@ ctladdr(cp)
 	ap->link = NULL;
 	/* While space: */
 	while (*cp != 0 && (*cp == ' ' || *cp == '\t')) ++cp;
+	/* CHANNEL: */
 	ap->channel = cp;
 	/* While not space: */
 	while (*cp != 0 && (*cp != ' ' && *cp != '\t')) ++cp;
 	*cp++ = '\0';
 	/* While space: */
 	while (*cp != 0 && (*cp == ' ' || *cp == '\t')) ++cp;
+	/* HOST: */
 	ap->host = cp;
 	ap->routermxes = NULL;
 	if (*cp == '(')
 	  cp = routermxes(cp,ap);  /* It is a bit complex.. */
 	else {
 	  /* While not space: */
-	  while (*cp != 0 && (*cp != ' ' && *cp != '\t')) ++cp;
+	  cp = skip821address(cp);
 	  *cp++ = '\0';
 	}
 	/* While space: */
 	while (*cp != 0 && (*cp == ' ' || *cp == '\t')) ++cp;
 	ap->user = cp;
 	/* the user value is allowed to have embedded whitespace */
-	while (*cp != '\0')
-	  ++cp;
-	--cp;
-	while (*cp >= '0' && *cp <= '9')
-	  --cp;
-	if (*cp == '-')
-	  --cp;
-	ap->misc = cp+1;
-	while (*cp == ' ' || *cp == '\t')
-	  --cp;
-	*++cp = '\0';
-
+	cp = skip821address(cp);
+	*cp++ = '\0';
+	ap->misc = cp;
 	return ap;
 }
 
