@@ -6,7 +6,9 @@
 #include "hostenv.h"
 #include <stdio.h>
 
+#ifndef strchr
 extern char *strchr();
+#endif
 
 int
 emptyline(line, size)
