@@ -24,11 +24,13 @@ extern FILE *verboselog;
 
 
 extern void *emalloc();
-extern void *realloc();
+extern void *erealloc();
+
+#define DEFCHARSET "UNKNOWN-8BIT"
 
 int cvtspace_copy __((struct rcpt *rp));
 
-/* #define USE_NEW_MIME2_CODE /* TESTING! */ /* not yet ready.. */
+#define USE_NEW_MIME2_CODE /* TESTING! */ /* not yet ready.. */
 #ifdef USE_NEW_MIME2_CODE
 
 /*
@@ -135,11 +137,11 @@ header8bit2QPnextword(line, word, len, inside, start_encode, fold, hdr_offset)
  */
 
 static int
-header8bit2QP(line, defcharset, outp, sizep, fold)
+header8bit2QP(line, defcharset, outp, sizep, osizep, fold)
      unsigned char *line;
      const char *defcharset;
      unsigned char **outp;
-     int *sizep;
+     int *sizep, *osizep;
      int fold;
 {
 	unsigned char *ptr, *outptr, *outline;
@@ -165,15 +167,18 @@ header8bit2QP(line, defcharset, outp, sizep, fold)
 	ptr = line;
 #endif
 	while (*ptr != '\0') {
+	  int wlen;
 	  if (header8bit2QPnextword(&ptr, word, &len, &inside, tmp, fold, hdr_offset)) {
 	    lines++;
 	  }
-	  abslen += strlen(word);
+	  wlen = strlen(word);
+	  abslen += wlen;
 	  if (abslen >= *sizep) {
-	    *outp = realloc(*outp,abslen+2);
+	    *outp = erealloc(*outp,abslen+2);
 	    *sizep = abslen+2;
 	  }
-	  strcat(*outp, word);
+	  memcpy(*outp+*osizep, word, wlen+1);
+	  *osizep += wlen;
 	}
 #if 0
 	{
@@ -264,7 +269,7 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	/* New MIME-2 code by mta@sci.fi in use.. */
 
 	char **inhdr, **inhdr1;
-	char **outhdr;
+	char **outhdr, *s, **oh;
 	int   outcnt = 0;
 	char *sumstr = emalloc(1);
 	int   sumlen = 0;
@@ -280,40 +285,37 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	inhdr1 = inhdr;
 
 	while (inhdr && *inhdr) {
-	  char *nn;
 	  int thislen = strlen(*inhdr);
-
+	  
 	  if (**inhdr == ' ' || **inhdr == '\t') {
+	    char *nn;
 	    /* Continuation line.. */
-	    nn = realloc(sumstr,sumlen+thislen+2);
+	    nn = erealloc(sumstr,sumlen+thislen+2);
 	    if (!nn) return -1; /* AUTCH! */
-	    strcat(nn+sumlen,*inhdr);
-	    strcat(nn+sumlen+thislen,"\n");
+	    memcpy(nn+sumlen,*inhdr,thislen);
+	    strcpy(nn+sumlen+thislen,"\n");
 	    sumlen += thislen+1;
 	    sumstr = nn;
 	  } else {
 	    /* Not a continuation line, Some sort of 'first line' */
 	    if (sumlen > 0) {
 	      char *s, *p;
-	      int rc, size = 0;
+	      int rc, size = sumlen;
 	      char *outstr = emalloc(sumlen);
+	      int osize = 0;
 
-	      rc = header8bit2QP(sumstr,DEFCHARSET,&outstr,&size,1);
+	      rc = header8bit2QP(sumstr,DEFCHARSET,&outstr,&size,&osize,1);
 	      s = outstr;
-	      while (s && *s) {
-		p = strlen(s,'\n');
-		if (p) *p++ = 0;
-		size = strlen(s);
-		if (outhdr == NULL)
-		  outhdr = (char **)emalloc(sizeof(void*) * 2);
-		else
-		  outhdr = (char**)erealloc(outhdr,
-					    sizeof(void*) * (outcnt+2));
-		outhdr[outcnt] = (char*)emalloc(size+1);
-		strcpy(outhdr[outcnt],s);
-		++outcnt;
-		s = p;
-	      }
+
+	      if (outhdr == NULL)
+		outhdr = (char **)emalloc(sizeof(void*) * 2);
+	      else
+		outhdr = (char**)erealloc(outhdr,
+					  sizeof(void*) * (outcnt+2));
+	      outhdr[outcnt] = (char*)emalloc(osize+1);
+	      memcpy(outhdr[outcnt],s,osize+1);
+	      ++outcnt;
+	      outhdr[outcnt] = NULL;
 
 	      /* Clean up the results */
 	      sumlen = 0;
@@ -321,37 +323,36 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	    } /* sumlen > 0 */
 
 	    /* Collect the 'first line' */
-	    nn = realloc(sumstr,thislen+2);
-	    if (!nn) return -1; /* AUTCH! */
+	    sumstr = erealloc(sumstr,thislen+2);
+	    if (!sumstr) return -1; /* AUTCH! */
 	    strcpy(sumstr,*inhdr);
-	    sumlen = thislen;
-
+	    strcpy(sumstr+thislen,"\n");
+	    sumlen = thislen+1;
 	  }
+	  ++inhdr;
 	}
 
 	/* Process the last of the headers */
 
 	if (sumlen > 0) {
 	  char *s, *p;
-	  int rc, size = 0;
+	  int rc, size = sumlen;
 	  char *outstr = emalloc(sumlen);
+	  int osize = 0;
 
-	  rc = header8bit2QP(sumstr,DEFCHARSET,&outstr,&size,1);
+	  rc = header8bit2QP(sumstr,DEFCHARSET,&outstr,&size,&osize,1);
 	  s = outstr;
-	  while (s && *s) {
-	    p = strlen(s,'\n');
-	    if (p) *p++ = 0;
-	    size = strlen(s);
-	    if (outhdr == NULL)
-	      outhdr = (char **)emalloc(sizeof(void*) * 2);
-	    else
-	      outhdr = (char**)erealloc(outhdr,
-					sizeof(void*) * (outcnt+2));
-	    outhdr[outcnt] = (char*)emalloc(size+1);
-	    strcpy(outhdr[outcnt],s);
-	    ++outcnt;
-	    s = p;
-	  }
+
+	  size = strlen(s);
+	  if (outhdr == NULL)
+	    outhdr = (char **)emalloc(sizeof(void*) * 2);
+	  else
+	    outhdr = (char**)erealloc(outhdr,
+				      sizeof(void*) * (outcnt+2));
+	  outhdr[outcnt] = (char*)emalloc(size+1);
+	  strcpy(outhdr[outcnt],s);
+	  ++outcnt;
+	  outhdr[outcnt] = NULL;
 
 	  /* Clean up the results */
 	  sumlen = 0;
@@ -359,8 +360,42 @@ if(verboselog)fprintf(verboselog,"After processing: '%s'\n",newbuf);
 	} /* sumlen > 0 */
 	free(sumstr);
 
-	/* XX: replace the converted headers with  outhdr -set. */
-	    /* XX: resize (enlarge) the header array if need be */	
 
+	/* replace the converted headers with  outhdr -set. */
+	inhdr = outhdr;
+	oh = NULL;
+	outcnt = 0;
+	while (*inhdr) {
+	  char *s = *inhdr;
+	  while (s && *s) {
+	    char *p = strchr(s,'\n');
+	    int len;
+
+	    if (p) *p++ = 0;
+	    len = strlen(s);
+
+	    if (outcnt == 0)
+	      oh = (char**)emalloc(sizeof(void*) * 2);
+	    else
+	      oh = (char**)erealloc(oh,sizeof(void*) * (outcnt+2));
+	    /* -- copy -- */
+	    oh[outcnt] = (char*)emalloc(len+1);
+	    memcpy(oh[outcnt],s,len+1);
+	    oh[++outcnt] = NULL;
+	    /* -- Continuation line ? -- */
+	    if (p) s = p;
+	    else   s = NULL;
+	  }
+	  free(*inhdr);
+	  ++inhdr;
+	}
+	free(outhdr);
+
+	inhdr1 = inhdr = *(rp->newmsgheadercvt);
+	*(rp->newmsgheadercvt) = oh;
+	while (*inhdr)
+	  ctlfree(rp->desc,*inhdr++);
+	ctlfree(rp->desc,inhdr1);
+	return 0;
 #endif /* USE_NEW_MIME2_CODE */
 }
