@@ -135,9 +135,10 @@ struct rcpt *rp;
 	  probe = *(rp->newmsgheader);
 
 	  while (*probe) { /* Copy over */
-	    *ss = (char *)emalloc(strlen(*probe)+1);;
-	    if (!*ss) return 0; /* Out of memory ? */
-	    strcpy(*ss,*probe);
+	    int len = strlen(*probe)+1;
+	    *ss = (char *)emalloc(len);
+	    /* emalloc() exits at the 'out of memory' condition */
+	    memcpy(*ss,*probe,len);
 	    ++hdrcnt;
 	    ++probe;
 	    ++ss;
@@ -179,7 +180,11 @@ append_header(va_alist)
 #endif
 	linebuf[0] = 0;
 
+#ifdef HAVE_VSNPRINTF
+	vsnprintf(linebuf, sizeof(linebuf)-1, fmt, pvar);
+#else
 	vsprintf(linebuf, fmt, pvar);
+#endif
 	va_end(pvar);
 
 	linelen = strlen(linebuf);
