@@ -90,6 +90,8 @@ extern int wait();
 # include <maillock.h>
 #endif
 
+extern time_t time __((time_t *));
+
 #ifdef HAVE_DOTLOCK
 #include "dotlock.c"
 #endif
@@ -259,6 +261,7 @@ int keepatime = 1;		/* preserve mbox st_atime, for login etc. */
 int creatflag = 1;		/* attempt to create files that don't exist */
 int mime8bit = 0;		/* Has code which translates  MIME text/plain
 				   Quoted-Printable  to 8BIT.. */
+int keep_header8 = 0;		/* Headers can have 8-bit stuff ? */
 int convert_qp = 0;		/* Flag: We have a job.. */
 int conversion_prohibited = 0;	/* Flag: Under no circumstances touch on it.. */
 int is_mime    = 0;		/* Flag: Msg is MIME msg .. */
@@ -381,7 +384,7 @@ main(argc, argv)
 	errflg = 0;
 	logfile = NULL;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "abc:gh:l:rRSMV8")) != EOF) {
+	while ((c = getopt(argc, argv, "abc:gh:Hl:rRSMV8")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
@@ -418,6 +421,9 @@ main(argc, argv)
 	  case 'l':		/* log file */
 	    logfile = strdup(optarg);
 	    break;
+	  case 'H':
+	    keep_header8 = 1;
+	    break;
 	  case '8':
 	    mime8bit = 1;
 	    break;
@@ -580,6 +586,10 @@ process(dp)
 	time(&curtime);
 	ts = ctime(&curtime);
 
+	if (!keep_header8 && headers_need_mime2(dp->recipients)) {
+	  headers_to_mime2(dp->recipients,verboselog);
+	}
+
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 	  /* seek to message body start on each iteration */
 	  if (lseek(dp->msgfd, (off_t)dp->msgbodyoffset, L_SET) < 0L) {
@@ -996,7 +1006,7 @@ deliver(dp, rp, timestring)
 	  notaryreport(rp->addr->user,"delivery",
 		       "2.2.0 (delivery successfull)",
 		       "x-local; 250 (Delivered successfully)");
-	  DIAGNOSTIC(rp, EX_OK, (char *)NULL, 0);
+	  DIAGNOSTIC(rp, EX_OK, "Ok", 0);
 	  return;
 	}
 
@@ -1251,7 +1261,7 @@ deliver(dp, rp, timestring)
 	  notaryreport(rp->addr->user,"delivery",
 		       "2.2.0 (Delivered successfully)",
 		       "x-local; 250 (Delivered successfully)");
-	  DIAGNOSTIC(rp, EX_OK, (char *)NULL, 0);
+	  DIAGNOSTIC(rp, EX_OK, "Ok", 0);
 	}
 #if	defined(HAVE_SOCKET)
 	else
@@ -1331,9 +1341,18 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	fromuser = rp->addr->link->user;
 	if (*fromuser == 0 ||
 	    strcmp(rp->addr->link->channel, "error") == 0)
+	  fromuser = "";
+
+	{
+	  char **hdrs = has_header(rp,"Return-Path:");
+	  if (hdrs) delete_header(rp,hdrs);
+	  append_header(rp,"Return-Path: <%s>\n", fromuser);
+	}
+
+	if (*fromuser == 0)
 	  fromuser = "postmaster";
-	if (fprintf(fp, "%s%s %s",
-		    FROM_,fromuser,timestring) < 0
+
+	if (fprintf(fp, "%s%s %s", FROM_, fromuser, timestring) < 0
 	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
 	  notaryreport(rp->addr->link->user, "failed",
 		       "5.2.2 (Write to user's mailbox failed)",
@@ -1466,7 +1485,7 @@ program(dp, rp, timestring)
 	int i, pid, in[2], out[2];
 	uid_t uid;
 	gid_t gid;
-	char *env[20], buf[8192], *cp, *s;
+	char *env[20], buf[8192], *cp, *s, *cpe;
 	int status;
 	struct passwd *pw;
 	FILE *errfp;
@@ -1476,6 +1495,7 @@ program(dp, rp, timestring)
 	env[i++] = "SHELL=/bin/sh";
 	env[i++] = "IFS= \t\n";
 	cp = buf;
+	cpe = buf + sizeof(buf);
 	if ((s = getzenv("PATH")) == NULL)
 	  env[i++] = "PATH=/usr/bin:/bin:/usr/ucb";
 	else {
@@ -1517,19 +1537,24 @@ program(dp, rp, timestring)
 	cp += strlen(cp) + 1;
 	sprintf(cp, "MAILSHARE=%s", s);
 	env[i++] = cp;
+	cp += strlen(cp) + 1;
 	if (rp->orcpt) {
-	  cp += strlen(cp) + 1;
 	  sprintf(cp, "ORCPT=%s", rp->orcpt);
 	  env[i++] = cp;
+	  cp += strlen(cp) + 1;
 	}
 	if (dp->envid) {
-	  cp += strlen(cp) + 1;
 	  sprintf(cp, "ENVID=%s", dp->envid);
 	  env[i++] = cp;
+	  cp += strlen(cp) + 1;
 	}
 
 	env[i] = NULL;
 
+	if (cp >= cpe)
+	  /* OVERFLOWED THE 8kB BUFFER FOR THE NEW ENVIRONMENT VARIABLES! */
+	  exit(EX_SOFTWARE);
+
 	/* now we can fork off and run the command... */
 	if (pipe(in) < 0) {
 	  notaryreport(NULL,NULL,NULL,NULL);
@@ -2406,6 +2431,21 @@ find_return_receipt_hdr (rp)
     return(hdr);
 }
 
+static void encodeXtext(fp,str)
+FILE *fp;
+char *str;
+{
+	while (*str) {
+	  u_char c = *str;
+	  if ('!' <= c && c <= '~' && c != '+' && c != '=')
+	    putc(c,fp);
+	  else
+	    fprintf(fp,"+%02X",c);
+	  ++str;
+	}
+}
+
+
 static void
 return_receipt (dp, retrecptaddr)
 	struct ctldesc *dp;
@@ -2440,6 +2480,10 @@ return_receipt (dp, retrecptaddr)
 	rp = dp->recipients;
 
 	/* copy To: from return-receipt address */
+	fprintf(mfp, "todsn NOTIFY=NEVER ORCPT=rfc822;");
+	encodeXtext(mfp, retrecptaddr);
+	fprintf(mfp, "\nto <%s>\n",retrecptaddr);
+	fprintf(mfp, "env-end\n");
 	fprintf(mfp, "To: %s\n", retrecptaddr);
 
 	/* copy error message file itself */
