@@ -58,11 +58,12 @@ char uucpname[MAXHOSTNAMELEN+1];
 
 char	*progname;
 int	readalready = 0;	/* does buffer contain valid message data? */
-int	can_8bit = 0;		/* Can do 8-bit stuff! */
 int	mimeqpnarrow = 0;	/* Can't send TAB thru without MIME-QP */
 FILE	*verboselog = NULL;
 FILE	*logfp   = NULL;
 int	maxwidth = 0;
+int	can_8bit = 0;		/* Can do 8-bit stuff! */
+int	keep_header8 = 0;	/* Don't do "MIME-2" to the headers */
 
 int	D_alloc = 0;		/* Memory debugging */
 
@@ -132,6 +133,8 @@ struct exmapinfo {
 {	0,		NULL,				EX_TEMPFAIL	}
 };
 
+char myhostname[MAXHOSTNAMELEN+1];
+
 #ifdef	lint
 #undef	putc
 #define	putc	fputc
@@ -171,8 +174,9 @@ main(argc, argv)
 	  ++progname;
 	errflg = 0;
 	host = channel = NULL;
+	myhostname[0] = 0;
 	cf = NULL;
-	while ((c = getopt(argc, argv, "f:c:h:QvVw:8")) != EOF) {
+	while ((c = getopt(argc, argv, "f:c:h:HQvVw:8")) != EOF) {
 	  switch (c) {
 	  case 'f':
 	    cf = optarg;
@@ -196,6 +200,9 @@ main(argc, argv)
 	  case '8':
 	    can_8bit = 1;
 	    break;
+	  case 'H':
+	    keep_header8 = 1;
+	    break;
 	  case 'w':
 	    maxwidth = atoi(optarg);
 	    if (maxwidth < 0)
@@ -207,7 +214,7 @@ main(argc, argv)
 	}
 	if (errflg || optind != argc - 1 || host == channel) {
 	  fprintf(stderr,
-		  "Usage: %s [-V][-v][-8 | -Q][-f cfgfile][-w maxwidth][-c channel | -h host] mailer\n", argv[0]);
+		  "Usage: %s [-V][-v][-H][-8 | -Q][-f cfgfile][-w maxwidth][-c channel | -h host] mailer\n", argv[0]);
 	  exit(EX_USAGE);
 	}
 	mailer = argv[optind];
@@ -470,7 +477,50 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	free((char *)av);
 	/* ... having forked and set up the pipe, we quickly continue */
 
-	/* XX: BSMTP et.al. envelope formation here! */
+	/* BSMTP et.al. envelope formation here! */
+
+	if (mp->flags & MO_BSMTP) {
+
+	  if (mp->flags & MO_BESMTP)
+	    fprintf(tafp,"EHLO %s",myhostname);
+	  else
+	    fprintf(tafp,"HELO %s",myhostname);
+	  if (mp->flags & MO_CRLF) putc('\r',tafp);
+	  putc('\n',tafp);
+
+	  fprintf(tafp,"MAIL FROM:<%s>", startrp->addr->link->user);
+	  if (mp->flags & MO_BESMTP) {
+	      fprintf(tafp," SIZE=%ld",startrp->desc->msgsizeestimate);
+	    if (can_8bit)
+	      fprintf(tafp," BODY=8BITMIME");
+	  }
+	  if (mp->flags & MO_BEDSMTP) {
+	    if (startrp->desc->envid != NULL)
+	      fprintf(tafp," ENVID=%s",startrp->desc->envid);
+	    if (startrp->desc->dsnretmode != NULL)
+	      fprintf(tafp, " RET=%s", startrp->desc->dsnretmode);
+	  }
+	  if (mp->flags & MO_CRLF) putc('\r',tafp);
+	  putc('\n',tafp);
+	  for (rp = startrp; rp != endrp; rp = rp->next) {
+	    fprintf(tafp,"RCPT TO:<%s>",rp->addr->user);
+	    /* if (mp->flags & MO_BESMTP) { } */
+	    if (mp->flags & MO_BEDSMTP) {
+	      if (rp->notify)
+		fprintf(tafp," NOTIFY=%s",rp->notify);
+	      if (rp->orcpt)
+		fprintf(tafp," ORCPT=%s",rp->orcpt);
+	    }
+	    if (mp->flags & MO_CRLF) putc('\r',tafp);
+	    putc('\n',tafp);
+	  }
+
+	  fprintf(tafp,"DATA");
+	  if (mp->flags & MO_CRLF) putc('\r',tafp);
+	  putc('\n',tafp);
+	}
+
+	/* Now continue with inside stuff -- well, normal UUCP stuff */
 
 	if (mp->flags & (MO_UNIXFROM|MO_REMOTEFROM)) {
 	  char *timestring;
@@ -507,7 +557,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	}
 
 	convertmode = _CONVERT_NONE;
-	if (!conversion_prohibited)
+	if (!conversion_prohibited) {
 	  switch (content_kind) {
 	  case 0:		/* No MIME headers defined */
 	    if (!can_8bit && !ascii_clean) {
@@ -539,13 +589,20 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  default:		/* ?? should not happen.. */
 	    break;
 	  }
+	  
+	  if (!keep_header8 && headers_need_mime2(startrp)) {
+	    headers_to_mime2(startrp,verboselog);
+	  }
+	}
 
  	/* Add the "Return-Path:" is it is desired, but does not yet
 	   exist.. */
-	if (mp->flags & MO_RETURNPATH)
-	  if (!has_header(startrp,"Return-Path:"))
-	    append_header(startrp,"Return-Path: <%s>\n",
-			  startrp->addr->link->user);
+	if (mp->flags & MO_RETURNPATH) {
+	  char **hdrs = has_header(startrp,"Return-Path:");
+	  if (hdrs) delete_header(startrp, hdrs);
+	  append_header(startrp,"Return-Path: <%s>\n",
+			startrp->addr->link->user);
+	}
 	if (mp->flags & MO_CRLF)
 	  writeheaders(startrp, tafp, "\r\n", convertmode, maxwidth);
 	else
@@ -570,6 +627,14 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  wait(NULL);
 	  return;
 	}
+
+	if (mp->flags & MO_BSMTP) {
+	  if (mp->flags & MO_CRLF)
+	    fprintf(tafp,".\r\nQUIT\r\n");
+	  else
+	    fprintf(tafp,".\nQUIT\n");
+	}
+	
 	fclose(tafp);
 	close(out[1]);	/* paranoia */
 	if (fgets(buf, sizeof buf, errfp) == NULL)
@@ -607,7 +672,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  sprintf(cp, "[exit status %d", WEXITSTATUS(status));
 	  if (s)
 	    sprintf(cp+strlen(cp), " (%s)", s);
-	  sprintf(cp+strlen(cp), " of command: %s", mp->command);
+	  /* sprintf(cp+strlen(cp), " of command: %s", mp->command); */
 	  strcat(cp, "]");
 	}
 	if (verboselog)
@@ -1057,11 +1122,11 @@ readsmcf(file, mailer)
 	  case 'n':	m.flags &= ~MO_UNIXFROM;	break;
 	  case 'r':	m.flags |= MO_RFROMFLAG;	break;
 	  case 's':	m.flags |= MO_STRIPQUOTES;	break;
-	  case 'b':	m.flags |= MO_BSMTP;		break;
+	  case 'b':	m.flags |= (MO_BSMTP|MO_HIDDENDOT); break;
 	  case 'B':	if (m.flags & MO_BESMTP) /* -BB */
 			    m.flags |= MO_BEDSMTP;
 			else
-			    m.flags |= MO_BESMTP|MO_BSMTP;
+			    m.flags |= MO_BESMTP|MO_BSMTP|MO_HIDDENDOT;
 			break;
 	  case 'A':		/* arpanet-compatibility */
 	  case 'C':		/* canonicalize remote hostnames */
