@@ -6,8 +6,8 @@
 
 #define	RFC974		/* If BIND, check that TCP SMTP service is enabled */
 
-#define	TIMEOUT		(5*60)		/* timeout in seconds, per exchange */
-#define ALARM_BLOCKSIZE 1024
+#define	TIMEOUT		(5*60)	/* timeout in seconds, per exchange */
+#define ALARM_BLOCKSIZE (6*560) /* less than 'smtp_bufsize' */
 #define ALARM_DOTTOOK   (60*60)
 			/* RFC 1123 recommends:
 			     - Initial 220: 5 minutes
@@ -178,7 +178,6 @@ int force_8bit = 0;		/* Claim to the remote to be 8-bit system, even
 int force_7bit = 0;		/* and reverse the previous.. */
 int keep_header8 = 0;		/* Don't do "MIME-2" to the headers */
 int checkwks = 0;
-int headers_contain_8bit = 0;
 FILE *logfp = NULL;
 extern int nobody;
 int rcpt_limit = 100;		/* Number of recipients that can be sent on
@@ -599,8 +598,6 @@ main(argc, argv)
 	    continue;
 	  }
 
-	  headers_contain_8bit = 0;
-
 	  if (dp->verbose) {
 	    verboselog = (FILE *)fopen(dp->verbose,"a");
 	    if (verboselog)
@@ -2537,8 +2534,7 @@ int smtp_sync(smtpfp,verboselog,r)
 	    }
 	    /* Diagnose the errors, we report successes AFTER the DATA phase.. */
 	    notarystatsave(s,NULL);
-	    notaryreport(pipercpts[idx]->addr->user,FAILED,
-			 status,NULL);
+	    notaryreport(pipercpts[idx]->addr->user,FAILED,status,NULL);
 	    diagnostic(pipercpts[idx],rc, 0, "%s", remotemsg);
 	  } else {
 	    /* Ok results */
