@@ -12,13 +12,17 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #ifdef HAVE_NDBM_H
+#define datum Ndatum
 #include <ndbm.h>
-#else
-#ifdef HAVE_GDBM_H
-#include <gdbm.h>
-#else
-	error:error:Only NDBM and GDBM supported! (NDBM primarily)
+#undef datum
 #endif
+#ifdef HAVE_GDBM_H
+#define datum Gdatum
+#include <gdbm.h>
+#undef datum
+#endif
+#ifdef HAVE_DB_H
+#include <db.h>
 #endif
 
 #define PROG "makendbm"
@@ -37,12 +41,27 @@ usage(prog,err,errno)
 char *prog, *err;
 int errno;
 {
-  fprintf(stderr,"Usage: %s database.name [infilename|-]\n",prog);
+  fprintf(stderr,"Usage: %s dbtype database.name [infilename|-]\n",prog);
+  fprintf(stderr,"  where supported dbtypes are:");
+#ifdef HAVE_NDBM_H
+  fprintf(stderr," ndbm");
+#endif
+#ifdef HAVE_GDBM_H
+  fprintf(stderr," gdbm");
+#endif
+#ifdef HAVE_DB_H
+  fprintf(stderr," btree");
+#endif
+  fprintf(stderr,"\n");
   fprintf(stderr,"  If no infilename is defined, database.name is assumed.\n");
 #ifdef HAVE_NDBM_H
   fprintf(stderr,"  (NDBM appends  .pag, and .dir  into actual db file names..)\n");
-#else /* GDBM.. */
+#endif
+#ifdef HAVE_GDBM_H
   fprintf(stderr,"  (GDBM appends .pag, to the actual db file name..)\n");
+#endif
+#ifdef HAVE_DB_H
+  fprintf(stderr,"  (BTREE appends .db, to the actual db file name..)\n");
 #endif
   fprintf(stderr," Error now: %s",err);
   if (errno > 0 && errno < sys_nerr)
@@ -51,67 +70,27 @@ int errno;
   exit (1);
 }
 
-int
-main(argc,argv)
-int argc;
-char *argv[];
-{
-  char *dbasename = NULL;
-  FILE *infile = NULL;
-#ifdef HAVE_NDBM_H
-  DBM *dbmfile;
-#else
-  GDBM_FILE dbmfile;
-#endif
-
-  if (argc < 2) usage(argv[0],"too few arguments",0);
-  if (argc > 3) usage(argv[0],"too many arguments",0);
-  dbasename = argv[1];
-
-  if (argc == 3) {
-    if (strcmp(argv[2],"-")==0)
-      infile = stdin;
-    else
-      infile = (FILE*)fopen(argv[2],"r");
-  } else
-    infile = (FILE*)fopen(argv[1],"r");
-
-  if (infile == NULL) usage(argv[0],"bad infile",errno);
-
-#ifdef HAVE_NDBM_H
-  dbmfile = dbm_open(dbasename, O_RDWR|O_CREAT|O_TRUNC, 0644);
-#else
-  /* Play loose .. don't do syncs while writing */
-  dbasename = strcpy(malloc(strlen(dbasename)+8),dbasename);
-  strcat(dbasename,".pag"); /* ALWAYS append this */
-  dbmfile = gdbm_open(dbasename, 0, GDBM_NEWDB|GDBM_FAST, 0644);
-#endif
-  if (dbmfile == NULL)
-    usage(argv[0],"Can't open dbase file",errno);
-
-
-  create_dbase(infile,dbmfile);
-
-  dbm_close(dbmfile);
-
-  return 0;
-}
 
 void
-create_dbase(infile,dbmfile)
-FILE *infile;
-#ifdef HAVE_NDBM_H
-DBM *dbmfile;
-#else
-GDBM_FILE dbmfile;
-#endif
+create_dbase(infile,dbf,typ)
+     FILE *infile;
+     void *dbf;
+     int typ;
 {
 	char linebuf[2048];
 	char *s, *t;
-	datum dat;
-	datum key;
+#ifdef HAVE_NDBM_H
+	DBM *ndbmfile = dbf;
+#endif
+#ifdef HAVE_GDBM_H
+	GDBM_FILE gdbmfile = dbf;
+#endif
+#ifdef HAVE_DB_H
+	DB *dbfile = dbf;
+#endif
 	int rc;
 
+
 	while (!feof(infile) && !ferror(infile)) {
 	  *linebuf = 0;
 	  fgets(linebuf,sizeof(linebuf)-1,infile);
@@ -137,15 +116,35 @@ GDBM_FILE dbmfile;
 	  }
 
 
-	  key.dptr  = t;
-	  key.dsize = strlen(t)+1;
-	  dat.dptr  = s;
-	  dat.dsize = strlen(s)+1;
-
 #ifdef HAVE_NDBM_H
-	  rc = dbm_store(dbmfile, key, dat, DBM_INSERT);
-#else
-	  rc = gdbm_store(dbmfile, key, dat, GDBM_INSERT);
+	  if (typ == 1) {
+	    Ndatum Ndat, Nkey;
+	    Nkey.dptr  = t;
+	    Nkey.dsize = strlen(t)+1;
+	    Ndat.dptr  = s;
+	    Ndat.dsize = strlen(s)+1;
+	    rc = dbm_store(ndbmfile, Nkey, Ndat, DBM_INSERT);
+	  }
+#endif
+#ifdef HAVE_GDBM_H
+	  if (typ == 2) {
+	    Gdatum Gdat, Gkey;
+	    Gkey.dptr  = t;
+	    Gkey.dsize = strlen(t)+1;
+	    Gdat.dptr  = s;
+	    Gdat.dsize = strlen(s)+1;
+	    rc = gdbm_store(gdbmfile, Gkey, Gdat, GDBM_INSERT);
+	  }
+#endif
+#ifdef HAVE_DB_H
+	  if (typ == 3) {
+	    DBT Bkey, Bdat;
+	    Bkey.data = t;
+	    Bkey.size = strlen(t)+1;
+	    Bdat.data = s;
+	    Bdat.size = strlen(t)+1;
+	    rc = (dbfile->put)(dbfile,&Bkey,&Bdat,R_NOOVERWRITE);
+	  }
 #endif
 	  if (rc > 0) {
 	    fprintf(stderr,"Key: `%s' is duplicate!\n",t);
@@ -154,3 +153,100 @@ GDBM_FILE dbmfile;
 	  }
 	}
 }
+
+
+int
+main(argc,argv)
+int argc;
+char *argv[];
+{
+	char *dbasename = NULL;
+	FILE *infile = NULL;
+	int typ = 0;
+#ifdef HAVE_NDBM_H
+	DBM *ndbmfile;
+#endif
+#ifdef HAVE_GDBM_H
+	GDBM_FILE gdbmfile;
+#endif
+#ifdef HAVE_DB_H
+	DB *dbfile;
+#endif
+	char *dbtype;
+	void *dbf;
+
+	if (argc < 3) usage(argv[0],"too few arguments",0);
+	if (argc > 4) usage(argv[0],"too many arguments",0);
+	dbasename = argv[1];
+
+	if (argc == 4) {
+	  if (strcmp(argv[3],"-")==0)
+	    infile = stdin;
+	  else
+	    infile = (FILE*)fopen(argv[3],"r");
+	} else
+	  infile = (FILE*)fopen(argv[2],"r");
+	dbtype = argv[1];
+
+	if (infile == NULL) usage(argv[0],"bad infile",errno);
+
+	typ = 0;
+#ifdef HAVE_NDBM_H
+	if (strcasecmp(dbtype,"ndbm")==0)
+	  typ = 1;
+	else
+#endif
+#ifdef HAVE_GDBM_H
+	  if (strcasecmp(dbtype,"gdbm")==0)
+	    typ = 2;
+	  else
+#endif
+#ifdef HAVE_DB_H
+	    if (strcasecmp(dbtype,"btree")==0)
+	      typ = 3;
+#endif
+	if (typ == 0)
+	  usage(argv[0],"unknown dbtype",0);
+
+#ifdef HAVE_NDBM_H
+	if (typ == 1) {
+	  ndbmfile = dbm_open(dbasename, O_RDWR|O_CREAT|O_TRUNC, 0644);
+	  dbf = ndbmfile;
+	}
+#endif
+#ifdef HAVE_GDBM_H
+	if (typ == 2) {
+	  /* Play loose .. don't do syncs while writing */
+	  dbasename = strcpy(malloc(strlen(dbasename)+8),dbasename);
+	  strcat(dbasename,".pag"); /* ALWAYS append this */
+	  gdbmfile = gdbm_open(dbasename, 0, GDBM_NEWDB|GDBM_FAST, 0644, NULL);
+	  dbf = gdbmfile;
+	}
+#endif
+#ifdef HAVE_DB_H
+	if (typ == 1) {
+	  dbfile = dbopen(dbasename, O_RDWR|O_CREAT|O_TRUNC, 0644,
+			  DB_BTREE, NULL);
+	  dbf = dbfile;
+	}
+#endif
+	if (dbf == NULL)
+	  usage(argv[0],"Can't open dbase file",errno);
+
+	create_dbase(infile,dbf,typ);
+
+#ifdef HAVE_NDBM_H
+	if (typ == 1)
+	  dbm_close(ndbmfile);	
+#endif
+#ifdef HAVE_GDBM_H
+	if (typ == 2)
+	  gdbm_close(gdbmfile);
+#endif
+#ifdef HAVE_DB_H
+	if (typ == 3)
+	  (dbfile->close)(dbfile);
+#endif
+
+	return 0;
+}
