@@ -11,10 +11,10 @@
 #include <ctype.h>
 #include <sys/types.h>
 #include <fcntl.h>
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
 #include <ndbm.h>
 #else
-#ifdef HAVE_GDBM
+#ifdef HAVE_GDBM_H
 #include <gdbm.h>
 #else
 	error:error:Only NDBM and GDBM supported! (NDBM primarily)
@@ -34,7 +34,7 @@ char *prog, *err;
 int errno;
 {
   fprintf(stderr,"Usage: %s database.name key\n",prog);
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
   fprintf(stderr,"  (NDBM appends  .pag, and .dir  into actual db file names..)\n");
 #else /* GDBM.. */
   fprintf(stderr,"  (GDBM DOES NOT append .pag, nor .dir  into actual db file names..)\n");
@@ -55,7 +55,7 @@ char *argv[];
   FILE *infile = NULL;
   datum key;
   datum result;
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
   DBM *dbmfile;
 #else
   GDBM_FILE dbmfile;
@@ -63,7 +63,7 @@ char *argv[];
 
   if (argc != 3) usage(argv[0],"wrong number of arguments",0);
   dbasename = argv[1];
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
   dbmfile = dbm_open(dbasename, O_RDONLY, 0644);
 #else
   dbmfile = gdbm_open(dbasename, 0, GDBM_READER, 0);
@@ -74,7 +74,7 @@ char *argv[];
   key.dptr = argv[2];
   key.dsize = strlen(argv[2]) +1;
 
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
   result = dbm_fetch(dbmfile,key);
 #else
   result = gdbm_fetch(dbmfile,key);
@@ -85,7 +85,7 @@ char *argv[];
   }
   printf("siz:%d, dat: %s\n",result.dsize,result.dptr);
 
-#ifdef HAVE_NDBM
+#ifdef HAVE_NDBM_H
   dbm_close(dbmfile);
 #else
   gdbm_close(dbmfile);
